//
// by Boris Stefanovic on 24/05/22
//

#include "distance.h"
#include <math.h>
#include "common.h"

#define ERROR -1.0


double distance_euclid_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim)return ERROR;
	int_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		int_t diff = p2->data[i] - p1->data[i];
		int_t item = diff * diff;
		acc += item;
	}
	return sqrt((double) acc);
}

int_t abs_diff(const int_t a1, const int_t a2) {
	int_t diff = a2 - a1;
	return diff >= 0 ? diff : -diff;
}

double distance_manhattan_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim)return ERROR;
	int_t acc = 0;
	for (size_t i = 0; i < p1->dim; ++i) {
		int_t diff = p2->data[i] - p1->data[i];
		int_t item = diff >= 0 ? diff : -diff;
		acc += item;
	}
	return (double) acc;
}

double distance_chebyshev_int(const vector_int_t* p1, const vector_int_t* p2) {
	if (p1->dim != p2->dim)return ERROR;
	int_t max = ERROR;
	int_t item;
	for (size_t i = 0; i < p1->dim; ++i) {
		item = abs_diff(p1->data[i], p2->data[i]);
		if (item > max) max = item;
	}
	return (double) max;
}
